@@ -13,7 +13,7 @@ import warnings
 warnings.filterwarnings("ignore")
 
 
-__version__ = '0.6.9'
+__version__ = '0.6.13'
 
 """
 Copyright 2018, Alexander J. Tarashansky, All rights reserved.
@@ -42,20 +42,15 @@ class SAM(object):
         Only use this argument if you want to pass in preloaded data. Otherwise
         use one of the load functions.
 
-    annotations : numpy.ndarray, optional, default None
-        A Numpy array of cell annotations.
-
 
     Attributes
     ----------
 
-    k: int
-        The number of nearest neighbors to identify for each cell
-        when constructing the nearest neighbor graph.
+    preprocess_args: dict
+        Dictionary of arguments used for the 'preprocess_data' function.
 
-    distance: str
-        The distance metric used when constructing the cell-to-cell
-        distance matrix.
+    run_args: dict
+        Dictionary of arguments used for the 'run' function.
 
     adata_raw: AnnData
         An AnnData object containing the raw, unfiltered input data.
@@ -65,7 +60,7 @@ class SAM(object):
 
     """
 
-    def __init__(self, counts=None, annotations=None):
+    def __init__(self, counts=None, inplace = False):
 
         if isinstance(counts, tuple) or isinstance(counts, list):
             raw_data, all_gene_names, all_cell_names = counts
@@ -92,26 +87,23 @@ class SAM(object):
             all_gene_names=np.array(list(counts.var_names))
             self.adata_raw = counts
 
-
-
         elif counts is not None:
             raise Exception(
                 "\'counts\' must be either a tuple/list of "
                 "(data,gene IDs,cell IDs) or a Pandas DataFrame of"
                 "cells x genes")
 
-        if(annotations is not None):
-            annotations = np.array(list(annotations))
-            if counts is not None:
-                self.adata_raw.obs['annotations'] = pd.Categorical(annotations)
-
         if(counts is not None):
             if(np.unique(all_gene_names).size != all_gene_names.size):
                 self.adata_raw.var_names_make_unique()
             if(np.unique(all_cell_names).size != all_cell_names.size):
                 self.adata_raw.obs_names_make_unique()
-
-            self.adata = self.adata_raw.copy()
+            
+            if inplace:
+                self.adata = self.adata_raw
+            else:
+                self.adata = self.adata_raw.copy()
+                
             if 'X_disp' not in self.adata_raw.layers.keys():
                   self.adata.layers['X_disp'] = self.adata.X
 
@@ -119,8 +111,6 @@ class SAM(object):
         self.preprocess_args = {}
 
     def preprocess_data(self, div=1, downsample=0, sum_norm=None,
-                        include_genes=None, exclude_genes=None,
-                        include_cells=None, exclude_cells=None,
                         norm='log', min_expression=1, thresh=0.01,
                         filter_genes=True):
         """Log-normalizes and filters the expression data.
@@ -151,26 +141,6 @@ class SAM(object):
             experimental and should only be used for raw, un-normalized UMI
             datasets). If None, the data is not normalized.
 
-        include_genes : array-like of string, optional, default None
-            A vector of gene names that specifies the genes to keep.
-            All other genes will be filtered out. Gene names are case-
-            sensitive.
-
-        exclude_genes : array-like of string, optional, default None
-            A vector of gene names that specifies the genes to
-            exclude. These genes will be filtered out. Gene names are case-
-            sensitive.
-
-        include_cells : array-like of string, optional, default None
-            A vector of cell names that specifies the cells to keep.
-            All other cells will be filtered out. Cell names are
-            case-sensitive.
-
-        exclude_cells : array-like of string, optional, default None
-            A vector of cell names that specifies the cells to exclude.
-            Thses cells will be filtered out. Cell names are
-            case-sensitive.
-
         min_expression : float, optional, default 1
             The threshold above which a gene is considered
             expressed. Gene expression values less than 'min_expression' are
@@ -182,18 +152,13 @@ class SAM(object):
             expressed if its expression value exceeds 'min_expression'.
 
         filter_genes : bool, optional, default True
-            Setting this to False turns off filtering operations. Genes passed
-            in exclude_genes or not passed in include_genes will still be
-            filtered.
+            Setting this to False turns off filtering operations.
 
         """
 
         self.preprocess_args = {
                 'div':div,
-                'downsample':downsample,
                 'sum_norm':sum_norm,
-                'include_genes':include_genes,
-                'exclude_genes':exclude_genes,
                 'norm':norm,
                 'min_expression':min_expression,
                 'thresh':thresh,
@@ -208,34 +173,6 @@ class SAM(object):
         except AttributeError:
             print('No data loaded')
 
-        # filter cells
-        cell_names = np.array(list(self.adata_raw.obs_names))
-        idx_cells = np.arange(D.shape[0])
-        if(include_cells is not None):
-            include_cells = np.array(list(include_cells))
-            idx2 = np.where(np.in1d(cell_names, include_cells))[0]
-            idx_cells = np.array(list(set(idx2) & set(idx_cells)))
-
-        if(exclude_cells is not None):
-            exclude_cells = np.array(list(exclude_cells))
-            idx4 = np.where(np.in1d(cell_names, exclude_cells,
-                                    invert=True))[0]
-            idx_cells = np.array(list(set(idx4) & set(idx_cells)))
-
-        if downsample > 0:
-            numcells = int(D.shape[0] / downsample)
-            rand_ind = np.random.choice(np.arange(D.shape[0]),
-                                        size=numcells, replace=False)
-            idx_cells = np.array(list(set(rand_ind) & set(idx_cells)))
-        else:
-            numcells = D.shape[0]
-
-        mask_cells = np.zeros(D.shape[0], dtype='bool')
-        mask_cells[idx_cells] = True
-
-        if mask_cells.sum() < mask_cells.size:
-            self.adata = self.adata_raw[mask_cells,:].copy()
-
         D = self.adata.X
         if isinstance(D,np.ndarray):
             D=sp.csr_matrix(D,dtype='float32')
@@ -303,19 +240,7 @@ class SAM(object):
         D.data[idx] = 0
 
         # filter genes
-        gene_names = np.array(list(self.adata.var_names))
         idx_genes = np.arange(D.shape[1])
-        if(include_genes is not None):
-            include_genes = np.array(list(include_genes))
-            idx = np.where(np.in1d(gene_names, include_genes))[0]
-            idx_genes = np.array(list(set(idx) & set(idx_genes)))
-
-        if(exclude_genes is not None):
-            exclude_genes = np.array(list(exclude_genes))
-            idx3 = np.where(np.in1d(gene_names, exclude_genes,
-                                    invert=True))[0]
-            idx_genes = np.array(list(set(idx3) & set(idx_genes)))
-
         if(filter_genes):
             a, ct = np.unique(D.indices, return_counts=True)
             c = np.zeros(D.shape[1])
@@ -342,7 +267,7 @@ class SAM(object):
         self.adata.uns['preprocess_args'] = self.preprocess_args
 
     def load_data(self, filename, transpose=True,
-                  save_sparse_file=None, sep=',', **kwargs):
+                  save_sparse_file=None, sep=',', calculate_avg=True,**kwargs):
         """Loads the specified data file. The file can be a table of
         read counts (i.e. '.csv' or '.txt'), with genes as rows and cells
         as columns by default. The file can also be a pickle file (output from
@@ -360,16 +285,20 @@ class SAM(object):
             assumes the input table is delimited by commas.
 
         save_sparse_file - str, optional, default None
-            Path to which the sparse data will be saved. If ending with '.h5ad',
-            writes the SAM 'adata_raw' object to a h5ad file (the native AnnData
-            file format) for faster loading in the future. If 'p', pickles the
-            (sparse data structure, gene names, cell names) tuple for faster
-            loading in the future.
+            Path to which the sparse data will be saved ('.h5ad').
+            Writes the SAM 'adata_raw' object to a h5ad file (the native AnnData
+            file format) for faster loading in the future.
 
         transpose - bool, optional, default True
             By default, assumes file is (genes x cells). Set this to False if
             the file has dimensions (cells x genes).
-
+           
+        calculate_avg - bool, optional, default True
+            If nearest neighbors are already calculated in the .h5ad file,
+            setting this parameter to True performs knn averaging and stores
+            the result in adata.layers['X_knn_avg']. This is a fairly dense
+            matrix, so set this to False if you do not need the averaged
+            expressions.
 
         """
         if filename.split('.')[-1] == 'p':
@@ -407,8 +336,20 @@ class SAM(object):
             self.adata.layers['X_disp'] = raw_data
 
         else:
-            self.adata_raw = anndata.read_h5ad(filename, **kwargs)
-            self.adata = self.adata_raw.copy()
+            self.adata = anndata.read_h5ad(filename, **kwargs)
+            if self.adata.raw is not None:
+                self.adata_raw = AnnData(X=self.adata.raw.X)
+                self.adata_raw.var_names = self.adata.raw.var_names
+                self.adata_raw.obs_names = self.adata.obs_names
+                self.adata_raw.obs = self.adata.obs
+                self.adata.raw = None
+                if ('X_knn_avg' not in self.adata.layers.keys()
+                    and 'neighbors' in self.adata.uns.keys() and
+                    calculate_avg):
+                    self.dispersion_ranking_NN();
+            else:
+                self.adata_raw = self.adata
+
             if 'X_disp' not in list(self.adata.layers.keys()):
                 self.adata.layers['X_disp'] = self.adata.X
             save_sparse_file = None
@@ -417,28 +358,10 @@ class SAM(object):
             if(save_sparse_file.split('.')[-1] == 'p'):
                 self.save_sparse_data(save_sparse_file)
             elif(save_sparse_file.split('.')[-1] == 'h5ad'):
-                self.save_anndata(save_sparse_file, data = 'adata_raw')
+                self.save_anndata(save_sparse_file)
 
-    def save_sparse_data(self, fname):
-        """Saves the tuple (raw_data,all_cell_names,all_gene_names) in a
-        Pickle file.
 
-        Parameters
-        ----------
-        fname - string
-            The filename of the output file.
-
-        """
-        data = self.adata_raw.X.T
-        if data.getformat()=='csr':
-            data=data.tocsc()
-
-        cell_names = np.array(list(self.adata_raw.obs_names))
-        gene_names = np.array(list(self.adata_raw.var_names))
-
-        pickle.dump((data, cell_names, gene_names), open(fname, 'wb'))
-
-    def save_anndata(self, fname, data = 'adata', **kwargs):
+    def save_anndata(self, fname, save_knn=False, **kwargs):
         """Saves `adata_raw` to a .h5ad file (AnnData's native file format).
 
         Parameters
@@ -446,9 +369,27 @@ class SAM(object):
         fname - string
             The filename of the output file.
 
+        save_knn - bool, optional, default = False
+            If True, saves `.layers['X_knn_avg']`. If False, does not save
+            this layer. Default value is set to False as the nearest-neighbor
+            averaged expression values can be quite dense.
+
         """
-        x = self.__dict__[data]
+        if not save_knn:
+            try:
+                Xknn = self.adata.layers['X_knn_avg']
+                del self.adata.layers['X_knn_avg']
+            except:
+                0;
+        x = self.adata
+        x.raw = self.adata_raw
+
         x.write_h5ad(fname, **kwargs)
+        x.raw = None
+        try:
+            self.adata.layers['X_knn_avg'] = Xknn
+        except:
+            0;
 
     def load_var_annotations(self, aname, sep=',', key_added = 'annotations'):
         """Loads gene annotations.
@@ -509,6 +450,27 @@ class SAM(object):
     def scatter(self, projection=None, c=None, cmap='rainbow', linewidth=0.0,
                 edgecolor='k', axes=None, colorbar=True, s=10, **kwargs):
 
+        """Display a scatter plot.
+        Displays a scatter plot using the SAM projection or another input
+        projection.
+
+        Parameters
+        ----------
+        projection - string, numpy.ndarray, default None
+            A case-sensitive string indicating the projection to display (a key
+            in adata.obsm) or a 2D numpy array with cell coordinates. If None,
+            projection defaults to UMAP.
+
+        c - string, numpy.ndarray, default None
+            Cell color values overlaid on the projection. Can be a string from adata.obs
+            to overlay cluster assignments / annotations or a 1D numpy array.
+
+        axes - matplotlib axis, optional, default None
+            Plot output to the specified, existing axes. If None, create new
+            figure window.
+
+        **kwargs - all keyword arguments in matplotlib.pyplot.scatter are eligible.
+        """
 
         try:
             import matplotlib.pyplot as plt
@@ -706,7 +668,7 @@ class SAM(object):
         Parameters
         ----------
         genes - numpy.array or list, default None
-            Genes for which contribution to each PC will be calculated. Set to None 
+            Genes for which contribution to each PC will be calculated. Set to None
             if you know ahead of time which PC you wish to remove from the data using
             'regress_genes'.
 
@@ -830,6 +792,9 @@ class SAM(object):
         if(k > D.shape[0] - 1):
             k = D.shape[0] - 2
 
+        if preprocessing not in ['StandardScaler','Normalizer',None,'None']:
+            raise ValueError('preprocessing must be \'StandardScaler\', \'Normalizer\', or None')
+
         self.run_args = {
                 'max_iter':max_iter,
                 'verbose':verbose,
@@ -928,15 +893,9 @@ class SAM(object):
                 print('Computing the t-SNE embedding...')
             self.run_tsne(**proj_kwargs)
         elif(projection == 'umap'):
-            if D.shape[0] < 100000:
-                if verbose:
-                    print('Computing the UMAP embedding...')
-                self.run_umap(**proj_kwargs)
-            else:
-                import scanpy.api as sc
-                self.adata.uns['neighbors']['connectivities'].setdiag(0)
-                self.adata.uns['neighbors']['connectivities'].eliminate_zeros()
-                sc.tl.umap(self.adata,min_dist=0.1)
+            if verbose:
+                print('Computing the UMAP embedding...')
+            self.run_umap(**proj_kwargs)
         elif(projection == 'diff_umap'):
             if verbose:
                 print('Computing the diffusion UMAP embedding...')
@@ -947,9 +906,6 @@ class SAM(object):
         if verbose:
             print('Elapsed time: ' + str(elapsed) + ' seconds')
 
-
-
-
     def calculate_nnm(
             self,
             D,
@@ -1007,29 +963,16 @@ class SAM(object):
         W = self.dispersion_ranking_NN(
             EDM, num_norm_avg=num_norm_avg)
 
-        self.adata.uns['X_processed'] = D_sub
+        self.adata.uns['X_processed'] = (D_sub,
+                              np.array(list(self.adata.var_names[gkeep])))
 
         return W, g_weighted, EDM
 
-    def _create_dict(self, exc):
-        pickle_dict = self.__dict__.copy()
-        if(exc is not None):
-            if 'ps' not in exc:
-                exc.append('ps')
-
-            for i in range(len(exc)):
-                try:
-                    del pickle_dict[exc[i]]
-                except NameError:
-                    0
-        return pickle_dict
-
     def run_tsne(self, X=None, metric='correlation', **kwargs):
         """Wrapper for sklearn's t-SNE implementation.
 
         See sklearn for the t-SNE documentation. All arguments are the same
-        with the exception that 'metric' is set to 'precomputed' by default,
-        implying that this function expects a distance matrix by default.
+        with the exception that 'metric' is set to 'correlation' by default.
         """
         if(X is not None):
             dt = man.TSNE(metric=metric, **kwargs).fit_transform(X)
@@ -1042,8 +985,6 @@ class SAM(object):
             tsne2d = dt
             self.adata.obsm['X_tsne'] = tsne2d
 
-    #def run_InPCA(self): #TODO
-
     def run_umap(self, X='X_pca', metric=None, **kwargs):
         """Wrapper for umap-learn.
 
@@ -1066,16 +1007,16 @@ class SAM(object):
             umap_obj = umap.UMAP(metric=metric, **kwargs)
             umap2d = umap_obj.fit_transform(X)
             self.adata.obsm['X_umap'] = umap2d
-
+            self.adata.uns['umap_obj'] = umap_obj
         else:
             umap_obj = umap.UMAP(metric=metric, **kwargs)
             dt = umap_obj.fit_transform(X)
-            return dt
+            return dt,umap_obj
 
     def run_diff_umap(self,use_rep='X_pca', metric='euclidean', n_comps=15,
                       method='gauss', **kwargs):
         """
-        Experimental -- running UMAP on the diffusion components
+        Experimental -- running UMAP on the diffusion components. Requires scanpy.
         """
         import scanpy.api as sc
         k = self.run_args.get('k',20)
@@ -1096,9 +1037,6 @@ class SAM(object):
 
     def run_diff_map(self,use_rep='X_pca', metric='euclidean', n_comps=15,
                      method = 'gauss', **kwargs):
-        """
-        Experimental -- running UMAP on the diffusion components
-        """
         import scanpy.api as sc
         k = self.run_args.get('k',20)
         distance = self.run_args.get('distance','correlation')
@@ -1276,7 +1214,7 @@ class SAM(object):
 
         from sklearn.cluster import KMeans
         if X is None:
-            D_sub = self.adata.uns['X_processed']
+            D_sub = self.adata.uns['X_processed'][0]
             X = ut.weighted_PCA(D_sub,npcs=npcs,do_weight=False)[0]
 
         km = KMeans(n_clusters = numc)
@@ -1332,7 +1270,7 @@ class SAM(object):
         import hdbscan
         if X is None:
             #X = self.adata.obsm['X_pca']
-            D = self.adata.uns['X_processed']
+            D = self.adata.uns['X_processed'][0]
             X = ut.weighted_PCA(D,npcs=npcs,do_weight=False)[0]
             X = Normalizer().fit_transform(X)
             save = True
@@ -1494,105 +1432,7 @@ class SAM(object):
 
         return markers
 
-    def identify_marker_genes_corr(self, labels=None, n_genes=4000):
-        """
-        Ranking marker genes based on their respective magnitudes in the
-        correlation dot products with cluster-specific reference expression
-        profiles.
-
-        Parameters
-        ----------
-
-        labels - numpy.array or str, optional, default None
-            Cluster labels to use for marker gene identification. If None,
-            assumes that one of SAM's clustering algorithms has been run. Can
-            be a string (i.e. 'louvain_clusters', 'kmeans_clusters', etc) to
-            specify specific cluster labels in adata.obs.
-
-        n_genes - int, optional, default 4000
-            By default, computes correlations on the top 4000 SAM-weighted genes.
-
-        """
-        if(labels is None):
-            try:
-                keys = np.array(list(self.adata.obs_keys()))
-                lbls = self.adata.obs[ut.search_string(
-                    keys, '_clusters')[0][0]].get_values()
-            except KeyError:
-                print("Please generate cluster labels first or set the "
-                      "'labels' keyword argument.")
-                return
-        elif isinstance(labels, str):
-            lbls = self.adata.obs[labels].get_values().flatten()
-        else:
-            lbls = labels
-
-
-        w=self.adata.var['weights'].values
-        s = StandardScaler()
-        idxg = np.argsort(-w)[:n_genes]
-        y1=s.fit_transform(self.adata.layers['X_disp'][:,idxg].A)*w[idxg]
-
-        all_gene_names = np.array(list(self.adata.var_names))[idxg]
-
-        markers = {}
-        lblsu=np.unique(lbls)
-        for i in lblsu:
-            Gcells = np.array(list(self.adata.obs_names[lbls==i]))
-            z1 = y1[np.in1d(self.adata.obs_names,Gcells),:]
-            m1 = (z1 - z1.mean(1)[:,None])/z1.std(1)[:,None]
-            ref = z1.mean(0)
-            ref = (ref-ref.mean())/ref.std()
-            g2 = (m1*ref).mean(0)
-            markers[i] = all_gene_names[np.argsort(-g2)]
-
-
-        self.adata.uns['marker_genes_corr'] = markers
-        return markers
-
-
-    def save(self, savename, dirname=None, exc=None):
-        """Saves all SAM attributes to a Pickle file.
-
-        Saves all SAM attributes to a Pickle file which can be later loaded
-        into an empty SAM object.
-
-        Parameters
-        ----------
-        savename - string
-            The name of the pickle file (not including the file extension) to
-            write to.
-
-        dirname - string, optional, default None
-            The path/name of the directory in which the Pickle file will be
-            saved. If None, the file will be saved to the current working
-            directory.
-
-        exc - array-like of strings, optional, default None
-            A vector of SAM attributes to exclude from the saved file. Use this
-            to exclude bulky objects that do not need to be saved.
-
-        """
-        pickle_dict = self.__dict__#self._create_dict(exc)
-
-        try:
-            del pickle_dict['ps']
-        except:
-            0;
-
-        if savename[-2:] != '.p':
-            savename = savename + '.p'
-
-        if(dirname is not None):
-            ut.create_folder(dirname + "/")
-            f = open(dirname + "/" + savename, 'wb')
-        else:
-            f = open(savename, 'wb')
-
-        pickle.dump(pickle_dict, f)
-        f.close()
-
-    def load(self, n):
+    def load(self, n, recalc_avg=True):
         """Loads SAM attributes from a Pickle file.
 
         Loads all SAM attributes from the specified Pickle file into the SAM
@@ -1609,3 +1449,6 @@ class SAM(object):
             self.__dict__[list(pick_dict.keys())[i]
                           ] = pick_dict[list(pick_dict.keys())[i]]
         f.close()
+
+        if recalc_avg:
+            self.dispersion_ranking_NN();
