@@ -10,7 +10,7 @@ import pandas as pd
 from ipyevents import Event
 from ipywidgets import Widget
 
-__version__ = '0.6.9'
+__version__ = '0.6.13'
 
 class SAMGUI(object):
 
@@ -164,6 +164,7 @@ class SAMGUI(object):
             slider = self.cs_box.children[12].children[1]
             slider.set_trait('min',slider.value)
             slider.set_trait('max',slider.value)
+
         else:
             f1 = go.FigureWidget()
             f1.add_scattergl(x=[], y=[]);
@@ -204,7 +205,7 @@ class SAMGUI(object):
             elif key == 'v':
                 self.reset_view(None);
             elif key == 'a':
-                self.cs_box.children[6].children[2].set_trait('value',not self.cs_box.children[6].children[2].value)
+                self.cs_box.children[9].children[2].set_trait('value',not self.cs_box.children[9].children[2].value)
 
     def close_tab(self,event):
         I = self.stab.selected_index
@@ -640,12 +641,15 @@ class SAMGUI(object):
         proj.observe(self.proj_update,'value')
 
         rs = widgets.VBox([widgets.HBox([runb,title]),
-                           widgets.HBox([dfts]),
+                           widgets.HBox([dfts,wpca]),
                            widgets.HBox([l3,knn]),
                            widgets.HBox([l4,nna]),
                            norm,
                            distance,
-                           proj])
+                           proj,
+                           widgets.HBox([l1,ngenes]),
+                           widgets.HBox([l2,npcs])])
+
         return rs
     def set_run_defaults(self,event):
         self.run_args = self.run_args_init.copy()
@@ -656,9 +660,9 @@ class SAMGUI(object):
         #,knn
         #,nna
         #norm,dist,proj
-        #wpca = self.rs_box.children[1].children[1]
-        #ngenes = self.rs_box.children[2].children[1]
-        #npcs = self.rs_box.children[3].children[1]
+        wpca = self.rs_box.children[1].children[1]
+        ngenes = self.rs_box.children[7].children[1]
+        npcs = self.rs_box.children[8].children[1]
         knn = self.rs_box.children[2].children[1]
         nna = self.rs_box.children[3].children[1]
         rnorm = self.rs_box.children[4]
@@ -670,15 +674,15 @@ class SAMGUI(object):
         init = self.run_args.get('k',20)
         knn.set_trait('value',init)
 
-        #init = self.run_args.get('npcs',150)
-        #if init is None:
-        #    init = 150;
-        #npcs.set_trait('value',init)
+        init = self.run_args.get('npcs',150)
+        if init is None:
+            init = 150;
+        npcs.set_trait('value',init)
 
-        #init = self.run_args.get('n_genes',3000)
-        #if init is None:
-        #    init = 3000;
-        #ngenes.set_trait('value',init)
+        init = self.run_args.get('n_genes',3000)
+        if init is None:
+            init = 3000;
+        ngenes.set_trait('value',init)
 
         init = self.run_args.get('preprocessing','Normalizer')
         if init is None:
@@ -689,9 +693,19 @@ class SAMGUI(object):
         init = self.run_args.get('projection','umap')
         proj.set_trait('value',init)
 
-        #init = self.run_args.get('weight_PCs',True)
-        #wpca.set_trait('value',init)
-        #self.run_args['weight_PCs'] = init
+        init = self.run_args.get('weight_PCs',True)
+        wpca.set_trait('value',init)
+        self.run_args['weight_PCs'] = init
+    def sam_weights(self,event):
+        s = self.sams[self.stab.selected_index]
+        self.marker_genes[self.stab.selected_index] = np.array(list(s.adata.var_names[np.argsort(-s.adata.var['weights'].get_values())]))
+        self.marker_genes_tt[self.stab.selected_index] = 'Genes ranked by SAM weights.'
+        self.cs_box.children[11].children[0].set_trait('tooltip',self.marker_genes_tt[self.stab.selected_index])
+
+        if self.cs_box.children[11].children[1].value != 0:
+            self.cs_box.children[11].children[1].set_trait('value',0)
+        else:
+            self.show_expression(str(self.marker_genes[self.stab.selected_index][0]))
 
     def weightpcs(self,event):
         t = self.run_args.get('weight_PCs',True)
@@ -787,7 +801,7 @@ class SAMGUI(object):
 
         cp = widgets.Button(
                description = 'Compute projection',
-               tooltip = 'Compute a 2D projection using the selected method in the dropdown menu to the left.',
+               tooltip = 'Compute a 2D projection using the selected method in the dropdown menu to the right.',
             layout={'width':'40%'}
         )
         cp.on_click(self.compute_projection)
@@ -807,7 +821,7 @@ class SAMGUI(object):
         )
         dp = widgets.Button(
                description = 'Display projection',
-               tooltip = 'Display the 2D projection selected in the dropdown menu to the left.',
+               tooltip = 'Display the 2D projection selected in the dropdown menu to the right.',
             layout={'width':'40%'}
         )
         dp.on_click(self.display_projection)
@@ -821,7 +835,7 @@ class SAMGUI(object):
         clm.observe(self.rewire_cluster_slider,'value')
         cl = widgets.Button(
                description = 'Cluster',
-               tooltip = 'Cluster the data using the selected method in the dropdown menu to the left.',
+               tooltip = 'Cluster the data using the selected method in the dropdown menu to the right.',
             layout={'width':'40%'}
         )
         cl.on_click(self.cluster_data)
@@ -853,7 +867,7 @@ class SAMGUI(object):
         )
         da = widgets.Button(
                description = 'Disp obs ann',
-               tooltip = 'Overlay the annotations selected in the dropdown menu to the left.',
+               tooltip = 'Overlay the annotations selected in the dropdown menu to the right.',
             layout={'width':'40%'}
         )
         da.on_click(self.display_annotation)
@@ -877,15 +891,23 @@ class SAMGUI(object):
         irm = widgets.Button(
                description = 'Find markers (RF)',
                tooltip = 'Identify marker genes of selected cells using a random forest classifier.',
-            layout={'width':'40%'}
+            layout={'width':'34%'}
         )
         irm.on_click(self.irm_genes)
         ism = widgets.Button(
                description = 'Find markers (SW)',
                tooltip = 'Identify marker genes of selected cells by finding genes with large SAM weights among the selected cells.',
-            layout={'width':'40%'}
+            layout={'width':'33%'}
         )
         ism.on_click(self.ism_genes)
+
+        sws = widgets.Button(
+               description = 'SAM ranking',
+               tooltip = 'Ranks genes by SAM weights.',
+            layout={'width':'33%'}
+        )
+        sws.on_click(self.sam_weights)
+
         us = widgets.Button(
                description = 'Unselect all (x)',
                tooltip = 'Unselect all cells. Pressing \'x\' on the keyboard while hovering over the scatter plot will do the same thing.',
@@ -906,11 +928,18 @@ class SAMGUI(object):
         usa.on_click(self.select_all)
         res.on_click(self.reset_view)
 
-        avg = widgets.Checkbox(
+        avg = widgets.Checkbox(indent=False,
                value=True,
                description = 'Avg expr',
-            layout={'width':'50%'}
+            layout={'width':'20%'}
         )
+
+        log = widgets.Checkbox(indent=False,
+               value=False,
+               description = 'Log colorbar',
+            layout={'width':'20%'}
+        )
+
         lann = widgets.Button(
             description='Annotate',
             tooltip = ('Enter the key of the \'obs\' annotation vector you wish to modify.'
@@ -937,7 +966,7 @@ class SAMGUI(object):
         )
         gsm = widgets.Text(
             value = '',
-            layout={'width':'70%'}
+            layout={'width':'50%'}
         )
         gsm.on_submit(self.get_similar_genes)
         lshg = widgets.Button(
@@ -948,7 +977,7 @@ class SAMGUI(object):
         )
         shg = widgets.Text(
             value = '',
-            layout={'width':'70%'}
+            layout={'width':'50%'}
         )
         shg.on_submit(self.show_expression)
 
@@ -960,7 +989,7 @@ class SAMGUI(object):
         rgenes = widgets.IntSlider(
             value=0,
             min=0,
-            max=50,
+            max=200,
             step=1,
             disabled=False,
             continuous_update=False,
@@ -1030,7 +1059,7 @@ class SAMGUI(object):
             disabled=True,
             layout={'width':'30%'})
         aslider = widgets.FloatSlider(
-            value=0.1,
+            value=0.05,
             min=0.,
             max=1,
             step=0.01,
@@ -1079,11 +1108,11 @@ class SAMGUI(object):
             widgets.HBox([l,cslider]),
             widgets.HBox([da,dam,acc]),
             widgets.HBox([dv,dvm]),
-            widgets.HBox([irm,ism,avg]),
+            widgets.HBox([irm,ism,sws]),
             widgets.HBox([us,usa,res]),
             widgets.HBox([lann,anno_name,anno]),
-            widgets.HBox([lgsm,gsm]),
-            widgets.HBox([lshg,shg]),
+            widgets.HBox([lgsm,gsm,avg]),
+            widgets.HBox([lshg,shg,log]),
             widgets.HBox([lgenes,rgenes]),
             widgets.HBox([lthr,thr]),
             widgets.HBox([lsf,sf]),
@@ -1092,18 +1121,18 @@ class SAMGUI(object):
             widgets.HBox([close,hotkeys])
         ])
     def reset_view(self,event):
-        self.create_plot(self.stab.selected_index,self.stab.get_title(self.stab.selected_index))
-
+        i=self.stab.selected_index
+        self.create_plot(i,self.stab.get_title(i))
+        self.marker_genes[i] = np.array(list(self.sams[i].adata.var_names))[np.argsort(-self.sams[i].adata.var['weights'].get_values())]
+        self.marker_genes_tt[i] = 'Genes ranked by SAM weights.'
+        self.cs_box.children[11].children[0].set_trait('tooltip',self.marker_genes_tt[i])
 
     def save_data(self,path):
         path=path.value
         if path != '':
             if path.split('.')[-1] == 'h5ad':
                 s = self.sams[self.stab.selected_index]
-                s.save_anndata(path,data='adata')
-            elif path.split('.')[-1] == 'p':
-                s = self.sams[self.stab.selected_index]
-                s.save(path)
+                s.save_anndata(path)
             elif (path.split('.')[-1] == 'png' or path.split('.')[-1] == 'pdf'
                   or path.split('.')[-1] == 'eps' or path.split('.')[-1] == 'jpg'):
                 if len(path.split('/'))>1:
@@ -1120,27 +1149,27 @@ class SAMGUI(object):
         if gene != '':
 
             try:
-                genes = ut.search_string(np.array(list(s.adata.var_names)),gene, case_sensitive=True)[0]
+                if not (gene in s.adata.var_names):
+                    genes = ut.search_string(np.array(list(s.adata.var_names)),gene, case_sensitive=True)[0]
+                    if genes is not -1:
+                        gene=genes[0]
 
-                if genes is not -1:
-                    gene=genes[0]
-
-                if self.cs_box.children[6].children[2].value:
+                if self.cs_box.children[9].children[2].value:
                     x = s.adata[:,gene].layers['X_knn_avg']
                     if sp.issparse(x):
                         a = x.A.flatten()
                     else:
                         a = x.flatten()
                 else:
-                    x = s.adata_raw[:,gene].X
+                    x = s.adata_raw[:,gene][s.adata.obs_names,:].X
                     if sp.issparse(x):
                         a = x.A.flatten()
                     else:
                         a = x.flatten()
 
-                if(self.cs_box.children[6].children[2].value):
+                if(self.cs_box.children[9].children[2].value):
                     if a.sum() == 0:
-                        x = s.adata_raw[:,gene].X
+                        x = s.adata_raw[:,gene][s.adata.obs_names,:].X
                         if sp.issparse(x):
                             a = x.A.flatten()
                         else:
@@ -1169,9 +1198,8 @@ class SAMGUI(object):
                 else:
                     title=gene
 
-                self.select_all(None)
+                #self.select_all(None)
                 self.update_colors_expr(a,title)
-                self.gene_expressions[self.stab.selected_index] = a
             except IndexError:
                 0; # do nothing
 
@@ -1303,6 +1331,10 @@ class SAMGUI(object):
             self.update_colors_anno(labels)
 
     def update_colors_expr(self,a,title):
+        if self.cs_box.children[10].children[-1].value:
+            a=np.log2(a+1)
+        self.gene_expressions[self.stab.selected_index] = a
+
         f1 = self.stab.children[self.stab.selected_index]
         f1.update_traces(marker = dict(color = a,colorscale='spectral',reversescale=True,
                                        showscale=True,colorbar_ticks='outside',
